@@ -15,13 +15,14 @@ from docutils import nodes
 from docutils.parsers.rst import directives, roles
 
 from sphinx import addnodes
+from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
 from sphinx.util import docutils
-from sphinx.util.docfields import DocFieldTransformer
+from sphinx.util.docfields import DocFieldTransformer, TypedField
 from sphinx.util.docutils import SphinxDirective
 
 if False:
     # For type annotation
-    from typing import Any, Dict  # NOQA
+    from typing import Any, Dict, Tuple  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.util.docfields import Field  # NOQA
     from sphinx.util.typing import DirectiveOption  # NOQA
@@ -66,6 +67,23 @@ class ObjectDescription(SphinxDirective):
     objtype = None          # type: str
     indexnode = None        # type: addnodes.index
 
+    # Warning: this might be removed in future version. Don't touch this from extensions.
+    _doc_field_type_map = {}  # type: Dict[str, Tuple[Field, bool]]
+
+    def get_field_type_map(self):
+        # type: () -> Dict[str, Tuple[Field, bool]]
+        if self._doc_field_type_map == {}:
+            for field in self.doc_field_types:
+                for name in field.names:
+                    self._doc_field_type_map[name] = (field, False)
+
+                if field.is_typed:
+                    typed_field = cast(TypedField, field)
+                    for name in typed_field.typenames:
+                        self._doc_field_type_map[name] = (field, True)
+
+        return self._doc_field_type_map
+
     def get_signatures(self):
         # type: () -> List[str]
         """
@@ -148,7 +166,7 @@ class ObjectDescription(SphinxDirective):
         node['objtype'] = node['desctype'] = self.objtype
         node['noindex'] = noindex = ('noindex' in self.options)
 
-        self.names = []  # type: List[str]
+        self.names = []  # type: List[Any]
         signatures = self.get_signatures()
         for i, sig in enumerate(signatures):
             # add a signature node for each signature in the current unit
@@ -186,10 +204,6 @@ class ObjectDescription(SphinxDirective):
         return [self.indexnode, node]
 
 
-# backwards compatible old name
-DescDirective = ObjectDescription
-
-
 class DefaultRole(SphinxDirective):
     """
     Set the default interpreted text role.  Overridden from docutils.
@@ -242,7 +256,6 @@ class DefaultDomain(SphinxDirective):
         self.env.temp_data['default_domain'] = self.env.domains.get(domain_name)
         return []
 
-# import all directives sphinx provides (for compatibility)
 from sphinx.directives.code import (  # noqa
     Highlight, CodeBlock, LiteralInclude
 )
@@ -254,6 +267,32 @@ from sphinx.directives.patches import (  # noqa
     Figure, Meta
 )
 
+deprecated_alias('sphinx.directives',
+                 {
+                     'Highlight': Highlight,
+                     'CodeBlock': CodeBlock,
+                     'LiteralInclude': LiteralInclude,
+                     'TocTree': TocTree,
+                     'Author': Author,
+                     'Index': Index,
+                     'VersionChange': VersionChange,
+                     'SeeAlso': SeeAlso,
+                     'TabularColumns': TabularColumns,
+                     'Centered': Centered,
+                     'Acks': Acks,
+                     'HList': HList,
+                     'Only': Only,
+                     'Include': Include,
+                     'Class': Class,
+                     'Figure': Figure,
+                     'Meta': Meta,
+                 },
+                 RemovedInSphinx40Warning)
+
+
+# backwards compatible old name (will be marked deprecated in 3.0)
+DescDirective = ObjectDescription
+
 
 def setup(app):
     # type: (Sphinx) -> Dict[str, Any]
