@@ -10,6 +10,7 @@
 
 import html
 import warnings
+from functools import partial
 
 from pygments import highlight
 from pygments.filters import ErrorToken
@@ -32,21 +33,21 @@ if False:
     # For type annotation
     from typing import Any, Dict  # NOQA
     from pygments.formatter import Formatter  # NOQA
+    from pygments.style import Style  # NOQA
 
 
 logger = logging.getLogger(__name__)
 
-lexers = {
-    'none': TextLexer(stripnl=False),
-    'python': PythonLexer(stripnl=False),
-    'python3': Python3Lexer(stripnl=False),
-    'pycon': PythonConsoleLexer(stripnl=False),
-    'pycon3': PythonConsoleLexer(python3=True, stripnl=False),
-    'rest': RstLexer(stripnl=False),
-    'c': CLexer(stripnl=False),
+lexers = {}  # type: Dict[str, Lexer]
+lexer_classes = {
+    'none': partial(TextLexer, stripnl=False),
+    'python': partial(PythonLexer, stripnl=False),
+    'python3': partial(Python3Lexer, stripnl=False),
+    'pycon': partial(PythonConsoleLexer, stripnl=False),
+    'pycon3': partial(PythonConsoleLexer, python3=True, stripnl=False),
+    'rest': partial(RstLexer, stripnl=False),
+    'c': partial(CLexer, stripnl=False),
 }  # type: Dict[str, Lexer]
-for _lexer in lexers.values():
-    _lexer.add_filter('raiseonerror')
 
 
 escape_hl_chars = {ord('\\'): '\\PYGZbs{}',
@@ -69,16 +70,8 @@ class PygmentsBridge:
     def __init__(self, dest='html', stylename='sphinx', trim_doctest_flags=None):
         # type: (str, str, bool) -> None
         self.dest = dest
-        if stylename is None or stylename == 'sphinx':
-            style = SphinxStyle
-        elif stylename == 'none':
-            style = NoneStyle
-        elif '.' in stylename:
-            module, stylename = stylename.rsplit('.', 1)
-            style = getattr(__import__(module, None, None, ['__name__']),
-                            stylename)
-        else:
-            style = get_style_by_name(stylename)
+
+        style = self.get_style(stylename)
         self.formatter_args = {'style': style}  # type: Dict[str, Any]
         if dest == 'html':
             self.formatter = self.html_formatter
@@ -91,6 +84,18 @@ class PygmentsBridge:
             warnings.warn('trim_doctest_flags option for PygmentsBridge is now deprecated.',
                           RemovedInSphinx30Warning, stacklevel=2)
 
+    def get_style(self, stylename):
+        # type: (str) -> Style
+        if stylename is None or stylename == 'sphinx':
+            return SphinxStyle
+        elif stylename == 'none':
+            return NoneStyle
+        elif '.' in stylename:
+            module, stylename = stylename.rsplit('.', 1)
+            return getattr(__import__(module, None, None, ['__name__']), stylename)
+        else:
+            return get_style_by_name(stylename)
+
     def get_formatter(self, **kwargs):
         # type: (Any) -> Formatter
         kwargs.update(self.formatter_args)
@@ -110,40 +115,55 @@ class PygmentsBridge:
             return '\\begin{Verbatim}[commandchars=\\\\\\{\\}]\n' + \
                    source + '\\end{Verbatim}\n'
 
-    def highlight_block(self, source, lang, opts=None, location=None, force=False, **kwargs):
-        # type: (str, str, Any, Any, bool, Any) -> str
-        if not isinstance(source, str):
-            source = source.decode()
+    def get_lexer(self, source, lang, opts=None, force=False, location=None):
+        # type: (str, str, Dict, bool, Any) -> Lexer
+        if not opts:
+            opts = {}
 
         # find out which lexer to use
         if lang in ('py', 'python'):
             if source.startswith('>>>'):
                 # interactive session
-                lexer = lexers['pycon']
+                lang = 'pycon'
             else:
-                lexer = lexers['python']
+                lang = 'python'
         elif lang in ('py3', 'python3', 'default'):
             if source.startswith('>>>'):
-                lexer = lexers['pycon3']
+                lang = 'pycon3'
             else:
-                lexer = lexers['python3']
+                lang = 'python3'
         elif lang == 'guess':
             try:
                 lexer = guess_lexer(source)
             except Exception:
                 lexer = lexers['none']
+
+        if lang in lexers:
+            lexer = lexers[lang]
+        elif lang in lexer_classes:
+            lexer = lexer_classes[lang](**opts)
         else:
-            if lang in lexers:
-                lexer = lexers[lang]
-            else:
-                try:
-                    lexer = lexers[lang] = get_lexer_by_name(lang, **(opts or {}))
-                except ClassNotFound:
-                    logger.warning(__('Pygments lexer name %r is not known'), lang,
-                                   location=location)
-                    lexer = lexers['none']
+            try:
+                if lang == 'guess':
+                    lexer = guess_lexer(lang, **opts)
                 else:
-                    lexer.add_filter('raiseonerror')
+                    lexer = get_lexer_by_name(lang, **opts)
+            except ClassNotFound:
+                logger.warning(__('Pygments lexer name %r is not known'), lang,
+                               location=location)
+                lexer = lexer_classes['none'](**opts)
+
+        if not force:
+            lexer.add_filter('raiseonerror')
+
+        return lexer
+
+    def highlight_block(self, source, lang, opts=None, force=False, location=None, **kwargs):
+        # type: (str, str, Dict, bool, Any, Any) -> str
+        if not isinstance(source, str):
+            source = source.decode()
+
+        lexer = self.get_lexer(source, lang, opts, force, location)
 
         # trim doctest options if wanted
         if isinstance(lexer, PythonConsoleLexer) and self.trim_doctest_flags:
@@ -164,7 +184,9 @@ class PygmentsBridge:
                                   'Highlighting skipped.'), lang,
                                type='misc', subtype='highlighting_failure',
                                location=location)
-            hlsource = highlight(source, lexers['none'], formatter)
+            lexer = self.get_lexer(source, 'none', opts, force, location)
+            hlsource = highlight(source, lexer, formatter)
+
         if self.dest == 'html':
             return hlsource
         else:
