@@ -1,39 +1,39 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util
     ~~~~~~~~~~~
 
     Utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
 
+import fnmatch
+import functools
 import os
+import posixpath
 import re
 import sys
-import fnmatch
 import tempfile
-import posixpath
 import traceback
 import unicodedata
+import warnings
+from codecs import BOM_UTF8
+from collections import deque
+from datetime import datetime
+from hashlib import md5
 from os import path
 from time import mktime, strptime
-from codecs import BOM_UTF8
-from datetime import datetime
-from collections import deque
+from urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
 
-from six import text_type, binary_type, itervalues
-from six.moves import range
-from six.moves.urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
 from docutils.utils import relative_path
 
+from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
 from sphinx.errors import PycodeError, SphinxParallelError, ExtensionError
+from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.console import strip_colors, colorize, bold, term_width_line  # type: ignore
 from sphinx.util.fileutil import copy_asset_file
-from sphinx.util.osutil import fs_encoding
 from sphinx.util import smartypants  # noqa
 
 # import other utilities; partly for backwards compatibility, so don't
@@ -48,7 +48,7 @@ from sphinx.util.matching import patfilter  # noqa
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Sequence, Set, Tuple, Union  # NOQA
+    from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Set, Tuple, Type, Union  # NOQA
 
 
 logger = logging.getLogger(__name__)
@@ -61,22 +61,20 @@ url_re = re.compile(r'(?P<schema>.+)://.*')     # type: Pattern
 # High-level utility functions.
 
 def docname_join(basedocname, docname):
-    # type: (unicode, unicode) -> unicode
+    # type: (str, str) -> str
     return posixpath.normpath(
         posixpath.join('/' + basedocname, '..', docname))[1:]
 
 
 def path_stabilize(filepath):
-    # type: (unicode) -> unicode
+    # type: (str) -> str
     "normalize path separater and unicode string"
     newpath = filepath.replace(os.path.sep, SEP)
-    if isinstance(newpath, text_type):
-        newpath = unicodedata.normalize('NFC', newpath)
-    return newpath
+    return unicodedata.normalize('NFC', newpath)
 
 
 def get_matching_files(dirname, exclude_matchers=()):
-    # type: (unicode, Tuple[Callable[[unicode], bool], ...]) -> Iterable[unicode]
+    # type: (str, Tuple[Callable[[str], bool], ...]) -> Iterable[str]
     """Get all file names in a directory, recursively.
 
     Exclude files and dirs matching some matcher in *exclude_matchers*.
@@ -85,13 +83,13 @@ def get_matching_files(dirname, exclude_matchers=()):
     dirname = path.normpath(path.abspath(dirname))
     dirlen = len(dirname) + 1    # exclude final os.path.sep
 
-    for root, dirs, files in walk(dirname, followlinks=True):
+    for root, dirs, files in os.walk(dirname, followlinks=True):
         relativeroot = root[dirlen:]
 
         qdirs = enumerate(path_stabilize(path.join(relativeroot, dn))
-                          for dn in dirs)  # type: Iterable[Tuple[int, unicode]]
+                          for dn in dirs)  # type: Iterable[Tuple[int, str]]
         qfiles = enumerate(path_stabilize(path.join(relativeroot, fn))
-                           for fn in files)  # type: Iterable[Tuple[int, unicode]]
+                           for fn in files)  # type: Iterable[Tuple[int, str]]
         for matcher in exclude_matchers:
             qdirs = [entry for entry in qdirs if not matcher(entry[1])]
             qfiles = [entry for entry in qfiles if not matcher(entry[1])]
@@ -103,12 +101,14 @@ def get_matching_files(dirname, exclude_matchers=()):
 
 
 def get_matching_docs(dirname, suffixes, exclude_matchers=()):
-    # type: (unicode, List[unicode], Tuple[Callable[[unicode], bool], ...]) -> Iterable[unicode]  # NOQA
+    # type: (str, List[str], Tuple[Callable[[str], bool], ...]) -> Iterable[str]  # NOQA
     """Get all file names (without suffixes) matching a suffix in a directory,
     recursively.
 
     Exclude files and dirs matching a pattern in *exclude_patterns*.
     """
+    warnings.warn('get_matching_docs() is now deprecated. Use get_matching_files() instead.',
+                  RemovedInSphinx40Warning)
     suffixpatterns = ['*' + s for s in suffixes]
     for filename in get_matching_files(dirname, exclude_matchers):
         for suffixpattern in suffixpatterns:
@@ -125,10 +125,10 @@ class FilenameUniqDict(dict):
     """
     def __init__(self):
         # type: () -> None
-        self._existing = set()  # type: Set[unicode]
+        self._existing = set()  # type: Set[str]
 
     def add_file(self, docname, newfile):
-        # type: (unicode, unicode) -> unicode
+        # type: (str, str) -> str
         if newfile in self:
             self[newfile][0].add(docname)
             return self[newfile][1]
@@ -138,12 +138,12 @@ class FilenameUniqDict(dict):
         while uniquename in self._existing:
             i += 1
             uniquename = '%s%s%s' % (base, i, ext)
-        self[newfile] = (set([docname]), uniquename)
+        self[newfile] = ({docname}, uniquename)
         self._existing.add(uniquename)
         return uniquename
 
     def purge_doc(self, docname):
-        # type: (unicode) -> None
+        # type: (str) -> None
         for filename, (docs, unique) in list(self.items()):
             docs.discard(docname)
             if not docs:
@@ -151,27 +151,61 @@ class FilenameUniqDict(dict):
                 self._existing.discard(unique)
 
     def merge_other(self, docnames, other):
-        # type: (Set[unicode], Dict[unicode, Tuple[Set[unicode], Any]]) -> None
+        # type: (Set[str], Dict[str, Tuple[Set[str], Any]]) -> None
         for filename, (docs, unique) in other.items():
             for doc in docs & set(docnames):
                 self.add_file(doc, filename)
 
     def __getstate__(self):
-        # type: () -> Set[unicode]
+        # type: () -> Set[str]
         return self._existing
 
     def __setstate__(self, state):
-        # type: (Set[unicode]) -> None
+        # type: (Set[str]) -> None
         self._existing = state
 
 
+class DownloadFiles(dict):
+    """A special dictionary for download files.
+
+    .. important:: This class would be refactored in nearly future.
+                   Hence don't hack this directly.
+    """
+
+    def add_file(self, docname, filename):
+        # type: (str, str) -> None
+        if filename not in self:
+            digest = md5(filename.encode()).hexdigest()
+            dest = '%s/%s' % (digest, os.path.basename(filename))
+            self[filename] = (set(), dest)
+
+        self[filename][0].add(docname)
+        return self[filename][1]
+
+    def purge_doc(self, docname):
+        # type: (str) -> None
+        for filename, (docs, dest) in list(self.items()):
+            docs.discard(docname)
+            if not docs:
+                del self[filename]
+
+    def merge_other(self, docnames, other):
+        # type: (Set[str], Dict[str, Tuple[Set[str], Any]]) -> None
+        for filename, (docs, dest) in other.items():
+            for docname in docs & set(docnames):
+                self.add_file(docname, filename)
+
+
 def copy_static_entry(source, targetdir, builder, context={},
                       exclude_matchers=(), level=0):
-    # type: (unicode, unicode, Any, Dict, Tuple[Callable, ...], int) -> None
+    # type: (str, str, Any, Dict, Tuple[Callable, ...], int) -> None
     """[DEPRECATED] Copy a HTML builder static_path entry from source to targetdir.
 
     Handles all possible cases of files, directories and subdirectories.
     """
+    warnings.warn('sphinx.util.copy_static_entry is deprecated for removal',
+                  RemovedInSphinx30Warning, stacklevel=2)
+
     if exclude_matchers:
         relpath = relative_path(path.join(builder.srcdir, 'dummy'), source)
         for matcher in exclude_matchers:
@@ -180,8 +214,7 @@ def copy_static_entry(source, targetdir, builder, context={},
     if path.isfile(source):
         copy_asset_file(source, targetdir, context, builder.templates)
     elif path.isdir(source):
-        if not path.isdir(targetdir):
-            os.mkdir(targetdir)
+        ensuredir(targetdir)
         for entry in os.listdir(source):
             if entry.startswith('.'):
                 continue
@@ -205,7 +238,7 @@ _DEBUG_HEADER = '''\
 
 
 def save_traceback(app):
-    # type: (Any) -> unicode
+    # type: (Any) -> str
     """Save the current exception's traceback in a temporary file."""
     import sphinx
     import jinja2
@@ -220,7 +253,7 @@ def save_traceback(app):
     last_msgs = ''
     if app is not None:
         last_msgs = '\n'.join(
-            '#   %s' % strip_colors(force_decode(s, 'utf-8')).strip()  # type: ignore
+            '#   %s' % strip_colors(s).strip()
             for s in app.messagelog)
     os.write(fd, (_DEBUG_HEADER %
                   (sphinx.__display_version__,
@@ -228,22 +261,20 @@ def save_traceback(app):
                    platform.python_implementation(),
                    docutils.__version__, docutils.__version_details__,
                    jinja2.__version__,  # type: ignore
-                   last_msgs)).encode('utf-8'))
+                   last_msgs)).encode())
     if app is not None:
-        for ext in itervalues(app.extensions):
+        for ext in app.extensions.values():
             modfile = getattr(ext.module, '__file__', 'unknown')
-            if isinstance(modfile, bytes):
-                modfile = modfile.decode(fs_encoding, 'replace')
             if ext.version != 'builtin':
                 os.write(fd, ('#   %s (%s) from %s\n' %
-                              (ext.name, ext.version, modfile)).encode('utf-8'))
-    os.write(fd, exc_format.encode('utf-8'))
+                              (ext.name, ext.version, modfile)).encode())
+    os.write(fd, exc_format.encode())
     os.close(fd)
     return path
 
 
 def get_module_source(modname):
-    # type: (str) -> Tuple[unicode, unicode]
+    # type: (str) -> Tuple[str, str]
     """Try to find the source code for a module.
 
     Can return ('file', 'filename') in which case the source is in the given
@@ -264,7 +295,9 @@ def get_module_source(modname):
             raise PycodeError('error getting filename for %r' % filename, err)
     if filename is None and loader:
         try:
-            return 'string', loader.get_source(modname)
+            filename = loader.get_source(modname)
+            if filename:
+                return 'string', filename
         except Exception as err:
             raise PycodeError('error getting source for %r' % modname, err)
     if filename is None:
@@ -277,13 +310,19 @@ def get_module_source(modname):
             filename += 'w'
     elif not (lfilename.endswith('.py') or lfilename.endswith('.pyw')):
         raise PycodeError('source is not a .py file: %r' % filename)
+    elif ('.egg' + os.path.sep) in filename:
+        pat = '(?<=\\.egg)' + re.escape(os.path.sep)
+        eggpath, _ = re.split(pat, filename, 1)
+        if path.isfile(eggpath):
+            return 'file', filename
+
     if not path.isfile(filename):
         raise PycodeError('source file is not present: %r' % filename)
     return 'file', filename
 
 
 def get_full_modname(modname, attribute):
-    # type: (str, unicode) -> unicode
+    # type: (str, str) -> str
     if modname is None:
         # Prevents a TypeError: if the last getattr() call will return None
         # then it's better to return it directly
@@ -306,11 +345,11 @@ _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 
 
 def detect_encoding(readline):
-    # type: (Callable) -> unicode
+    # type: (Callable[[], bytes]) -> str
     """Like tokenize.detect_encoding() from Py3k, but a bit simplified."""
 
     def read_or_stop():
-        # type: () -> unicode
+        # type: () -> bytes
         try:
             return readline()
         except StopIteration:
@@ -329,13 +368,13 @@ def detect_encoding(readline):
         return orig_enc
 
     def find_cookie(line):
-        # type: (unicode) -> unicode
+        # type: (bytes) -> str
         try:
             line_string = line.decode('ascii')
         except UnicodeDecodeError:
             return None
 
-        matches = _coding_re.findall(line_string)  # type: ignore
+        matches = _coding_re.findall(line_string)
         if not matches:
             return None
         return get_normal_name(matches[0])
@@ -359,9 +398,31 @@ def detect_encoding(readline):
     return default
 
 
+class UnicodeDecodeErrorHandler:
+    """Custom error handler for open() that warns and replaces."""
+
+    def __init__(self, docname):
+        # type: (str) -> None
+        self.docname = docname
+
+    def __call__(self, error):
+        # type: (UnicodeDecodeError) -> Tuple[Union[str, str], int]
+        linestart = error.object.rfind(b'\n', 0, error.start)
+        lineend = error.object.find(b'\n', error.start)
+        if lineend == -1:
+            lineend = len(error.object)
+        lineno = error.object.count(b'\n', 0, error.start) + 1
+        logger.warning(__('undecodable source characters, replacing with "?": %r'),
+                       (error.object[linestart + 1:error.start] + b'>>>' +
+                        error.object[error.start:error.end] + b'<<<' +
+                        error.object[error.end:lineend]),
+                       location=(self.docname, lineno))
+        return ('?', error.end)
+
+
 # Low-level utility functions and classes.
 
-class Tee(object):
+class Tee:
     """
     File-like object writing to two streams.
     """
@@ -371,7 +432,7 @@ class Tee(object):
         self.stream2 = stream2
 
     def write(self, text):
-        # type: (unicode) -> None
+        # type: (str) -> None
         self.stream1.write(text)
         self.stream2.write(text)
 
@@ -384,7 +445,7 @@ class Tee(object):
 
 
 def parselinenos(spec, total):
-    # type: (unicode, int) -> List[int]
+    # type: (str, int) -> List[int]
     """Parse a line number spec (such as "1,2,4-6") and return a list of
     wanted line numbers.
     """
@@ -398,10 +459,8 @@ def parselinenos(spec, total):
             elif len(begend) == 1:
                 items.append(int(begend[0]) - 1)
             elif len(begend) == 2:
-                start = int(begend[0] or 1)     # type: ignore
-                                                # left half open (cf. -10)
-                end = int(begend[1] or max(start, total))   # type: ignore
-                                                            # right half open (cf. 10-)
+                start = int(begend[0] or 1)  # left half open (cf. -10)
+                end = int(begend[1] or max(start, total))  # right half open (cf. 10-)
                 if start > end:  # invalid range (cf. 10-1)
                     raise ValueError
                 items.extend(range(start - 1, end))
@@ -414,15 +473,17 @@ def parselinenos(spec, total):
 
 
 def force_decode(string, encoding):
-    # type: (unicode, unicode) -> unicode
+    # type: (str, str) -> str
     """Forcibly get a unicode string out of a bytestring."""
-    if isinstance(string, binary_type):
+    warnings.warn('force_decode() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
+    if isinstance(string, bytes):
         try:
             if encoding:
                 string = string.decode(encoding)
             else:
                 # try decoding with utf-8, should only work for real UTF-8
-                string = string.decode('utf-8')
+                string = string.decode()
         except UnicodeError:
             # last resort -- can't fail
             string = string.decode('latin1')
@@ -430,21 +491,26 @@ def force_decode(string, encoding):
 
 
 class attrdict(dict):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn('The attrdict class is deprecated.',
+                      RemovedInSphinx40Warning, stacklevel=2)
+
     def __getattr__(self, key):
-        # type: (unicode) -> unicode
+        # type: (str) -> str
         return self[key]
 
     def __setattr__(self, key, val):
-        # type: (unicode, unicode) -> None
+        # type: (str, str) -> None
         self[key] = val
 
     def __delattr__(self, key):
-        # type: (unicode) -> None
+        # type: (str) -> None
         del self[key]
 
 
 def rpartition(s, t):
-    # type: (unicode, unicode) -> Tuple[unicode, unicode]
+    # type: (str, str) -> Tuple[str, str]
     """Similar to str.rpartition from 2.5, but doesn't return the separator."""
     i = s.rfind(t)
     if i != -1:
@@ -453,7 +519,7 @@ def rpartition(s, t):
 
 
 def split_into(n, type, value):
-    # type: (int, unicode, unicode) -> List[unicode]
+    # type: (int, str, str) -> List[str]
     """Split an index entry into a given number of parts at semicolons."""
     parts = [x.strip() for x in value.split(';', n - 1)]
     if sum(1 for part in parts if part) < n:
@@ -462,7 +528,7 @@ def split_into(n, type, value):
 
 
 def split_index_msg(type, value):
-    # type: (unicode, unicode) -> List[unicode]
+    # type: (str, str) -> List[str]
     # new entry types must be listed in directives/other.py!
     if type == 'single':
         try:
@@ -484,18 +550,18 @@ def split_index_msg(type, value):
 
 
 def format_exception_cut_frames(x=1):
-    # type: (int) -> unicode
+    # type: (int) -> str
     """Format an exception with traceback, but only the last x frames."""
     typ, val, tb = sys.exc_info()
     # res = ['Traceback (most recent call last):\n']
-    res = []  # type: List[unicode]
+    res = []  # type: List[str]
     tbres = traceback.format_tb(tb)
     res += tbres[-x:]
     res += traceback.format_exception_only(typ, val)
     return ''.join(res)
 
 
-class PeekableIterator(object):
+class PeekableIterator:
     """
     An iterator which wraps any iterable and makes it possible to peek to see
     what's the next item.
@@ -504,6 +570,8 @@ class PeekableIterator(object):
         # type: (Iterable) -> None
         self.remaining = deque()  # type: deque
         self._iterator = iter(iterable)
+        warnings.warn('PeekableIterator is deprecated.',
+                      RemovedInSphinx40Warning, stacklevel=2)
 
     def __iter__(self):
         # type: () -> PeekableIterator
@@ -528,63 +596,56 @@ class PeekableIterator(object):
     def peek(self):
         # type: () -> Any
         """Return the next item without changing the state of the iterator."""
-        item = next(self)  # type: ignore
+        item = next(self)
         self.push(item)
         return item
 
 
 def import_object(objname, source=None):
-    # type: (str, unicode) -> Any
+    # type: (str, str) -> Any
+    """Import python object by qualname."""
     try:
-        module, name = objname.rsplit('.', 1)
-    except ValueError as err:
-        raise ExtensionError('Invalid full object name %s' % objname +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
-    try:
-        return getattr(__import__(module, None, None, [name]), name)
-    except ImportError as err:
-        raise ExtensionError('Could not import %s' % module +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
-    except AttributeError as err:
-        raise ExtensionError('Could not find %s' % objname +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
+        objpath = objname.split('.')
+        modname = objpath.pop(0)
+        obj = __import__(modname)
+        for name in objpath:
+            modname += '.' + name
+            try:
+                obj = getattr(obj, name)
+            except AttributeError:
+                __import__(modname)
+                obj = getattr(obj, name)
+
+        return obj
+    except (AttributeError, ImportError) as exc:
+        if source:
+            raise ExtensionError('Could not import %s (needed for %s)' %
+                                 (objname, source), exc)
+        else:
+            raise ExtensionError('Could not import %s' % objname, exc)
 
 
 def encode_uri(uri):
-    # type: (unicode) -> unicode
-    split = list(urlsplit(uri))  # type: Any
+    # type: (str) -> str
+    split = list(urlsplit(uri))
     split[1] = split[1].encode('idna').decode('ascii')
-    split[2] = quote_plus(split[2].encode('utf-8'), '/').decode('ascii')
-    query = list((q, quote_plus(v.encode('utf-8')))
-                 for (q, v) in parse_qsl(split[3]))
-    split[3] = urlencode(query).decode('ascii')
+    split[2] = quote_plus(split[2].encode(), '/')
+    query = list((q, v.encode()) for (q, v) in parse_qsl(split[3]))
+    split[3] = urlencode(query)
     return urlunsplit(split)
 
 
-def split_docinfo(text):
-    # type: (unicode) -> Sequence[unicode]
-    docinfo_re = re.compile('\\A((?:\\s*:\\w+:.*?\n(?:[ \\t]+.*?\n)*)+)', re.M)
-    result = docinfo_re.split(text, 1)  # type: ignore
-    if len(result) == 1:
-        return '', result[0]
-    else:
-        return result[1:]
-
-
 def display_chunk(chunk):
-    # type: (Any) -> unicode
+    # type: (Any) -> str
     if isinstance(chunk, (list, tuple)):
         if len(chunk) == 1:
-            return text_type(chunk[0])
+            return str(chunk[0])
         return '%s .. %s' % (chunk[0], chunk[-1])
-    return text_type(chunk)
+    return str(chunk)
 
 
 def old_status_iterator(iterable, summary, color="darkgreen", stringify_func=display_chunk):
-    # type: (Iterable, unicode, str, Callable[[Any], unicode]) -> Iterator
+    # type: (Iterable, str, str, Callable[[Any], str]) -> Iterator
     l = 0
     for item in iterable:
         if l == 0:
@@ -600,10 +661,9 @@ def old_status_iterator(iterable, summary, color="darkgreen", stringify_func=dis
 # new version with progress info
 def status_iterator(iterable, summary, color="darkgreen", length=0, verbosity=0,
                     stringify_func=display_chunk):
-    # type: (Iterable, unicode, str, int, int, Callable[[Any], unicode]) -> Iterable  # NOQA
+    # type: (Iterable, str, str, int, int, Callable[[Any], str]) -> Iterable
     if length == 0:
-        for item in old_status_iterator(iterable, summary, color, stringify_func):
-            yield item
+        yield from old_status_iterator(iterable, summary, color, stringify_func)
         return
     l = 0
     summary = bold(summary)
@@ -620,7 +680,45 @@ def status_iterator(iterable, summary, color="darkgreen", length=0, verbosity=0,
         logger.info('')
 
 
+class SkipProgressMessage(Exception):
+    pass
+
+
+class progress_message:
+    def __init__(self, message):
+        # type: (str) -> None
+        self.message = message
+
+    def __enter__(self):
+        # type: () -> None
+        logger.info(bold(self.message + '... '), nonl=True)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        # type: (Type[Exception], Exception, Any) -> bool
+        if isinstance(exc_value, SkipProgressMessage):
+            logger.info(__('skipped'))
+            if exc_value.args:
+                logger.info(*exc_value.args)
+            return True
+        elif exc_type:
+            logger.info(__('failed'))
+        else:
+            logger.info(__('done'))
+
+        return False
+
+    def __call__(self, f):
+        # type: (Callable) -> Callable
+        @functools.wraps(f)
+        def wrapper(*args, **kwargs):
+            with self:
+                return f(*args, **kwargs)
+
+        return wrapper
+
+
 def epoch_to_rfc1123(epoch):
+    # type: (float) -> str
     """Convert datetime format epoch to RFC1123."""
     from babel.dates import format_datetime
 
@@ -630,37 +728,36 @@ def epoch_to_rfc1123(epoch):
 
 
 def rfc1123_to_epoch(rfc1123):
+    # type: (str) -> float
     return mktime(strptime(rfc1123, '%a, %d %b %Y %H:%M:%S %Z'))
 
 
 def xmlname_checker():
+    # type: () -> Pattern
     # https://www.w3.org/TR/REC-xml/#NT-Name
-    # Only Python 3.3 or newer support character code in regular expression
     name_start_chars = [
-        u':', [u'A', u'Z'], u'_',  [u'a', u'z'], [u'\u00C0', u'\u00D6'],
-        [u'\u00D8', u'\u00F6'], [u'\u00F8', u'\u02FF'], [u'\u0370', u'\u037D'],
-        [u'\u037F', u'\u1FFF'], [u'\u200C', u'\u200D'], [u'\u2070', u'\u218F'],
-        [u'\u2C00', u'\u2FEF'], [u'\u3001', u'\uD7FF'], [u'\uF900', u'\uFDCF'],
-        [u'\uFDF0', u'\uFFFD']]
-
-    if sys.version_info.major == 3:
-        name_start_chars.append([u'\U00010000', u'\U000EFFFF'])
+        ':', ['A', 'Z'], '_', ['a', 'z'], ['\u00C0', '\u00D6'],
+        ['\u00D8', '\u00F6'], ['\u00F8', '\u02FF'], ['\u0370', '\u037D'],
+        ['\u037F', '\u1FFF'], ['\u200C', '\u200D'], ['\u2070', '\u218F'],
+        ['\u2C00', '\u2FEF'], ['\u3001', '\uD7FF'], ['\uF900', '\uFDCF'],
+        ['\uFDF0', '\uFFFD'], ['\U00010000', '\U000EFFFF']]
 
     name_chars = [
-        u"\\-", u"\\.", [u'0', u'9'], u'\u00B7', [u'\u0300', u'\u036F'],
-        [u'\u203F', u'\u2040']
+        "\\-", "\\.", ['0', '9'], '\u00B7', ['\u0300', '\u036F'],
+        ['\u203F', '\u2040']
     ]
 
-    def convert(entries, splitter=u'|'):
+    def convert(entries, splitter='|'):
+        # type: (Any, str) -> str
         results = []
         for entry in entries:
             if isinstance(entry, list):
-                results.append(u'[%s]' % convert(entry, u'-'))
+                results.append('[%s]' % convert(entry, '-'))
             else:
                 results.append(entry)
         return splitter.join(results)
 
     start_chars_regex = convert(name_start_chars)
     name_chars_regex = convert(name_chars)
-    return re.compile(u'(%s)(%s|%s)*' % (
+    return re.compile('(%s)(%s|%s)*' % (
         start_chars_regex, start_chars_regex, name_chars_regex))
